@@ -10,16 +10,17 @@
 	Transformed back into standalone asm, with help of
 	gcc -S -DHAVE_CONFIG_H -I.  -march=pentium -O3 -Wall -pedantic -fno-strict-aliasing -DREAL_IS_FLOAT -c -o decode_mmxsse.{S,c}
 
-	The difference between SSE and 3DNowExt is the dct64 function and the synth function name.
+	The difference between SSE and 3DNowExt is the INT123_dct64 function and the synth function name.
 	This template here uses the SYNTH_NAME and MPL_DCT64 macros for this - see decode_sse.S and decode_3dnowext.S...
 	That's not memory efficient since there's doubled code, but it's easier than giving another function pointer.
 	Maybe I'll change it in future, but now I need something that works.
 
-	Original comment from MPlayer source follows:
+	Original comment from MPlayer source follows. Regarding the license history see
+	synth_mmx.S, which the original comment about this being licensed under GPL is
+	relating to.
 */
 
 /*
- * this code comes under GPL
  * This code was taken from http://www.mpg123.org
  * See ChangeLog of mpg123-0.59s-pre.1 for detail
  * Applied to mplayer by Nick Kurshev <nickols_k@mail.ru>
@@ -48,11 +49,25 @@ SYNTH_NAME:
 	pushl	%ebp
 /* stack:0=ebp 4=back 8=bandptr 12=channel 16=samples 20=buffs 24=bo 28=decwins */
 	movl	%esp, %ebp
+
 /* Now the old stack addresses are preserved via %epb. */
+#ifdef PIC
+	subl  $8,%esp /* What has been called temp before. */
+#else
 	subl  $4,%esp /* What has been called temp before. */
+#endif
 	pushl	%edi
 	pushl	%esi
 	pushl	%ebx
+
+#ifdef PIC
+	#undef _EBX_
+	#define _EBX_ %eax
+	GET_GOT
+#define EBXSAVE -4(%ebp)
+	movl _EBX_, EBXSAVE /* save PIC register */
+#endif
+
 #define TEMP 12(%esp)
 /* APP */
 	movl 12(%ebp),%ecx
@@ -94,6 +109,9 @@ SYNTH_NAME:
 	leal (%ecx,%ebx,2), %edx
 	movl (%esp),%ecx /* restore, but leave value on stack */
 	shrl $1, %ecx
+#ifdef PIC
+	movl EBXSAVE, _EBX_
+#endif
 	ALIGN16
 3:
 	movq  (%edx),%mm0
@@ -130,8 +148,8 @@ SYNTH_NAME:
 	packssdw %mm4,%mm4
 	movq	(%edi), %mm1
 	punpckldq %mm4, %mm0
-	pand   one_null, %mm1
-	pand   null_one, %mm0
+	pand   LOCAL_VAR(one_null), %mm1
+	pand   LOCAL_VAR(null_one), %mm0
 	por    %mm0, %mm1
 	movq   %mm1,(%edi)
 	leal 64(%esi),%esi
@@ -166,6 +184,10 @@ SYNTH_NAME:
 4:
 	subl $64,%esi
 	movl $7,%ecx
+
+#ifdef PIC
+	movl EBXSAVE, _EBX_
+#endif
 	ALIGN16
 5:
 	movq  (%edx),%mm0
@@ -206,8 +228,8 @@ SYNTH_NAME:
 	psubsw %mm5,%mm4
 	movq	(%edi), %mm1
 	punpckldq %mm4, %mm0
-	pand   one_null, %mm1
-	pand   null_one, %mm0
+	pand   LOCAL_VAR(one_null), %mm1
+	pand   LOCAL_VAR(null_one), %mm0
 	por    %mm0, %mm1
 	movq   %mm1,(%edi)
 	subl $64,%esi
@@ -241,6 +263,6 @@ SYNTH_NAME:
 	popl	%ebx
 	popl	%esi
 	popl	%edi
-	addl $4,%esp
+	mov		%ebp, %esp
 	popl	%ebp
 	ret
